;HErb JOhnson HRJ
;Simple program to display fixed message in ROM
;may not use RAM at all, it's that simple
;last edit Oct 10 2024
;
;RF = message pointer to ASCII display string
;RE = startburst table addr for ASCII char
;RD = starburst pattern from table for char
;RC.0 = bit digit pointer 80 40 20 10 08 04 02 01

RF	equ   15
RE	equ   14
RD	equ   13
RC	equ   12

CR	EQU   0DH		;CR as end of message

	org 0000H

RESET:	DIS
	DB   00H		;disable interrupts

START:	;set up msg pointer to start of msg
	LDI   HIGH MSG
	PHI   RF 
	LDI   LOW MSG
	PLO   RF

;set up digit pointer RC.0

	LDI   80H
	PLO   RC

;loop to display 8 chars ending with CR

NXTCHR: LDN   RF		;get next char to display
	SMI   CR		;is it end of message?
	BZ    START		;if so, restart display

; set up starburst table entry
	LDI   HIGH TABLE	;point to table page addr in RE.1
	PHI   RE		;RE.0 will be based on char value

	LDN   RF		;ASCII char to display 0-7FH
	SHL			;* 2 for word-size table address
	PLO   RE		;RE.0 becomes word offset into table

;get starburst pattern word from table

	LDA   RE		;grab high byte of pattern & advance
	PHI   RD		;into high byte of RD
	LDN   RE		;grab low byte of pattern
	PLO   RD		;into low byte of RD

;pattern in RD, becomes ROM "write" address to display

	GLO   RC		;digit bit in RC.0

;RC.0 is content "written" to ROM, selects display digit
;RD is "address" for ROM "write"
;write to ROM has 1ms wait delay
	
	STR   RD

;advance message char, digit pointer

	INC   RF		;to next char in msg
	GLO   RC		;advance digit pointer
	SHR			;to next digit on display
	PLO   RC   		;save it

	BR    NXTCHR		;display next char until CR end of msg 

;for this simple program, all messages are 8 ASCII chars and 9th is CR
MSG:	DB    "HELLO WD"
	DB    CR ; CR terminating char

;scrolling:
; instead of simple 8-character loop, could test RC.0 for zero
; (shift right past 8th digit),  increment a counter at RC.1
; 64 counts would be 64 * 8 "digits" * 1ms = .5 seconds
; then incr starting message address by 1
; Result: scroll message to right. 
; then when at end of message, restart everything.
;
; later, something similar at beginning of message, an 8-char blank buffer


;		ADDRESS LINES				
;
;	A A A A A A A A A A A A A A A A 
;	1 1 1 1 1 1 9 8 7 6 5 4 3 2 1 0
;	5 4 3 2 1 0
;
;		SEGMENTS
;	* G F E D C B A N M L K J H D G
;	  1  	                    P 2
;
; HRJ both A segmts are tied together. Both D segmts are tied together.
; * A15 not used, segments "written" in 0000-7FFFH ROM memory (15 bits)
;
; table is limited to 80H entries (ASCII chars 00H-7FH)
; so table can't be longer than 256D or 100H bytes
; and table start at XX00H beginning of ROM memory page
; so entry into table is simply high (TABLE) and low (2*ASCII value)

	ORG   0100H
TABLE:	;courtesy of David Madole 
;ascii 00H
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
;ascii 10H
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty
	DW    7FFFH		    ;empty	
;ASCII 20H = space
	DB    %01111111, %11111111    ; SPC
	DB    %01111001, %11111101    ; !
	DB    %01111101, %11110111    ; "
	DB    %00110001, %10110110    ; #
	DB    %00010010, %10110111    ; $
	DB    %00011011, %00000010    ; %
	DB    %00100110, %11010011    ; &
	DB    %01111111, %11110111    ; '
	DB    %01111111, %11001111    ; (
	DB    %01111111, %01101111    ; )
	DB    %00111111, %00000010    ; *
	DB    %00111111, %10110110    ; +
	DB    %01111111, %01111110    ; ,
	DB    %00111111, %11111110    ; -
	DB    %01111111, %11111101    ; .
	DB    %01111111, %01101111    ; /
;ascii 30H = zero
	DB    %01000000, %01101111    ; 0
	DB    %01111001, %11101111    ; 1
	DB    %00100100, %11111110    ; 2
	DB    %00110000, %11111110    ; 3
	DB    %00011001, %11111110    ; 4
	DB    %00010010, %11011111    ; 5
	DB    %00000010, %11111110    ; 6
	DB    %01111000, %11111111    ; 7
	DB    %00000000, %11111110    ; 8
	DB    %00010000, %11111110    ; 9
	DB    %01111111, %10110111    ; :
	DB    %01111111, %01110111    ; ;
	DB    %01111111, %11001111    ; <
	DB    %00110111, %10110110    ; =
	DB    %01111111, %01111010    ; >
	DB    %01111100, %10111100    ; ?
;ascii 40H = @
	DB    %01000100, %11110110    ; @
	DB    %00001000, %11111110    ; A
	DB    %01110000, %10110110    ; B
	DB    %01000110, %11111111    ; C
	DB    %01110000, %10110111    ; D
	DB    %00000110, %11111110    ; E
	DB    %00001110, %11111110    ; F
	DB    %01000010, %11111110    ; G
	DB    %00001001, %11111110    ; H
	DB    %01110110, %10110111    ; I
	DB    %01100001, %11111111    ; J
	DB    %00001111, %11001111    ; K
	DB    %01000111, %11111111    ; L
	DB    %01001001, %11101011    ; M
	DB    %01001001, %11011011    ; N
	DB    %01000000, %11111111    ; O
	DB    %00001100, %11111110    ; P
	DB    %01000000, %11011111    ; Q
	DB    %00001100, %11011110    ; R
	DB    %00010010, %11111110    ; S
	DB    %01111110, %10110111    ; T
	DB    %01000001, %11111111    ; U
	DB    %01001111, %01101111    ; V
	DB    %01001001, %01011111    ; W
	DB    %01111111, %01001011    ; X
	DB    %01111111, %10101011    ; Y
	DB    %01110110, %01101111    ; Z
	DB    %01000110, %11111111    ; [
	DB    %01111111, %11011011    ; \
	DB    %01110000, %11111111    ; ]
	DB    %01111111, %01011111    ; ^
	DB    %01110111, %11111111    ; _
;ascii 60H = ' and lower case a-z, will implement later


	END

